# TMAKEFLAT -- Run apflatten to make a 2-d pixel normalization file
# By Doug Mink, SAO Telescope Data Center
# September 20, 2007

procedure tmakeflat (input, flatfile)

string	input = ""	{prompt="Input multispec image to correct"}
string	flatfile=""	{prompt="Output pixel to pixel normalization file"}
string	apertures="1-102"	{prompt="Apertures to flatten"}
string	apref=""	{prompt="Aperture reference file"}
file	apdir="tres$data"	{prompt="Aperture reference directory"}

bool verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb
	file imin, imout, imroot, imflat, newfile, adir, aref
	int lroot
	string history

	imin = input
	vb = verbose
	imflat = flatfile
	adir = apdir
	fixdir (adir)
	adir = fixdir.newdir
	aref = apref

# Add extension to input file, if one is not present
	smatch (imin, ".fits")
	if (!smatch.found) {
	    imIn = imin // ".fits"
	    }

# Add FITS extension to normalization file if it is missing
	smatch (imflat, ".fits")
	if (!smatch.found) {
	    imflat = imflat // ".fits"
	    }
	if (vb) {
	    print ("MAKEFLAT: ", imin," -> ",imflat)
	    }

# Run apflatten
	apflatten.apertures = apertures
	apflatten.references = adir // aref
	apflatten.interactive = yes
	apflatten.find = yes
	apflatten.recenter = yes
	apflatten.resize = no
	apflatten.edit = yes
	apflatten.trace = yes
	apflatten.fittrace = yes
	apflatten.flatten = yes
	apflatten.fitspec = yes
	apflatten.line = INDEF
	apflatten.nsum = 1
	apflatten.threshold = 1.
	apflatten.pfit = "fit1d"
	apflatten.clean = no
	apflatten.saturation = INDEF
	apflatten.readnoise = "2.8"
	apflatten.gain = "1."
	apflatten.lsigma = 4.
	apflatten.usigma = 4.
	apflatten.function = "spline3"
	apflatten.order = 70
	apflatten.sample = "*"
	apflatten.naverage = 1
	apflatten.niterate = 0
	apflatten.low_reject = 3.
	apflatten.high_reject = 3.
	apflatten.grow = 0.
	apflatten (imin, imflat)

# Add history line including time to normalization file
	gettime ()
	history = gettime.date // " " // gettime.time // " pixel normalization file from" // imin
        hedit (imflat, "PIXFLAT", history, add+, verify-)

end
